#include <stdio.h>
#include <net/if.h>
#include <linux/sockios.h>

const char *stn[256] = {

"dot3StatsAlignmentErrors",
"dot3StatsFCSErrors",
"dot3StatsSingleCollisionFrames",
"dot3StatsMultipleCollisionFrames",
"dot3StatsSQETestErrors",
"dot3StatsDeferredTransmissions",
"dot3StatsLateCollisions",
"dot3StatsExcessiveCollisions",
"dot3StatsInternalMacTransmitErrors",
"dot3StatsCarrierSenseErrors",
"dot3StatsFrameTooLongs",
"dot3StatsInternalMacReceiveErrors",

"ifIndex",
"ifType",
"ifMtu",
"ifSpeed",
"ifAdminStatus",
"ifOperStatus",
"ifLastChange",
"ifInDiscards",
"ifInErrors",
"ifInUnknownProtos",
"ifOutDiscards",
"ifOutErrors",
"ifOutQLen",
"ifPhysAddress", NULL,
"ifDescr", NULL, NULL, NULL, NULL, NULL, NULL, NULL,
NULL,	/* Alignment. */
"ifHCInOctets", NULL,
"ifHCInUcastPkts", NULL,
"ifHCInMulticastPkts", NULL,
"ifHCInBroadcastPkts", NULL,
"ifHCOutOctets", NULL,
"ifHCOutUcastPkts", NULL,
"ifHCOutMulticastPkts", NULL,
"ifHCOutBroadcastPkts", NULL,
"ifLinkUpDownTrapEnable",
"ifHighSpeed",
"ifPromiscuousMode",
"ifConnectorPresent",

"nicCmdsSetHostState",
"nicCmdsFDRFiltering",
"nicCmdsSetRecvProdIndex",
"nicCmdsUpdateGencommStats",
"nicCmdsResetJumboRing",
"nicCmdsAddMCastAddr",
"nicCmdsDelMCastAddr",
"nicCmdsSetPromiscMode",
"nicCmdsLinkNegotiate",
"nicCmdsSetMACAddr",
"nicCmdsClearProfile",
"nicCmdsSetMulticastMode",
"nicCmdsClearStats",
"nicCmdsSetRecvJumboProdIndex",
"nicCmdsSetRecvMiniProdInde",
"nicCmdsRefreshStats",
"nicCmdsUnknown",

"nicEventsNICFirmwareOperational",
"nicEventsStatsUpdated",
"nicEventsLinkStateChanged",
"nicEventsError",
"nicEventsMCastListUpdated",
"nicEventsResetJumboRing",

"nicRingSetSendProdIndex",
"nicRingSetSendConsIndex",
"nicRingSetRecvReturnProdIndex",

"nicInterrupts",
"nicAvoidedInterrupts",

"nicEventThresholdHit",
"nicSendThresholdHit",
"nicRecvThresholdHit",

"nicDmaRdOverrun",
"nicDmaRdUnderrun",
"nicDmaWrOverrun",
"nicDmaWrUnderrun",
"nicDmaRdMasterAborts",
"nicDmaWrMasterAborts",

"nicDmaWriteRingFull",
"nicDmaReadRingFull",
"nicEventRingFull",
"nicEventProducerRingFull",
"nicTxMaxDescrRingFull",
"nicOutOfTxBufSpaceFrameRetry",
"nicNoMoreWrDMADescriptors",
"nicNoMoreRxBDs",
"nicNoSpaceInReturnRing",
"nicSendBDs",
"nicRecvBDs",
"nicJumboRecvBDs",
"nicMiniRecvBDs",
"nicTotalRecvBDs",
"nicTotalSendBDs",
"nicJumboSpillOver",
"nicSbusHangCleared",
"nicEnqEventDelayed",

"nicMacRxLateColls",
"nicMacRxLinkLostDuringPkt",
"nicMacRxPhyDecodeErr",
"nicMacRxMacAbort",
"nicMacTruncNoResources",
"nicMacRxDropUla",
"nicMacRxDropMcast",
"nicMacRxFlowControl",
"nicMacRxDropSpace",
"nicMacRxColls",
"nicMacRxTotalAttempts",
"nicMacRxLinkAttentions",
"nicMacRxLinkAttentions",
"nicMacRxSyncAttentions",
"nicMacRxConfigAttentions",
"nicMacReset",
"nicMacRxBufDescrAttns",
"nicMacRxBufAttns",
"nicMacRxZeroFrameCleanup",
"nicMacRxOneFrameCleanup",
"nicMacRxMultipleFrameCleanup",
"nicMacRxTimerCleanup",
"nicMacRxDmaCleanup",

"nicMacCollision[1]",
"nicMacCollision[2]",
"nicMacCollision[3]",
"nicMacCollision[4]",
"nicMacCollision[5]",
"nicMacCollision[6]",
"nicMacCollision[7]",
"nicMacCollision[8]",
"nicMacCollision[9]",
"nicMacCollision[10]",
"nicMacCollision[11]",
"nicMacCollision[12]",
"nicMacCollision[13]",
"nicMacCollision[14]",
"nicMacCollision[15]",
"nicMacTxTotalAttentions",

"nicProfile[0]",
"nicProfile[1]",
"nicProfile[2]",
"nicProfile[3]",
"nicProfile[4]",
"nicProfile[5]",
"nicProfile[6]",
"nicProfile[7]",
"nicProfile[8]",
"nicProfile[9]",
"nicProfile[10]",
"nicProfile[11]",
"nicProfile[12]",
"nicProfile[13]",
"nicProfile[14]",
"nicProfile[15]",

};

main(int argc, char **argv)
{
	int i;
	int fd = socket(AF_INET, SOCK_DGRAM, 0);
	struct ifreq ifr;
	unsigned int st[0x100];

	if (argc != 2)
		goto oops;

	strcpy(ifr.ifr_name, argv[1]);
	ifr.ifr_data = st;

	if (ioctl(fd, SIOCDEVPRIVATE+3, &ifr))
		goto oops;

	for (i=0; i<0x100; i++) {
		if (stn[i]) {
			if (strcmp("ifPhysAddress", stn[i]) == 0) {
				int k;
				st[i] = htonl(st[i]);
				st[i+1] = htonl(st[i+1]);
				printf("% 40s ", "ifPhysAddress");
				for (k=2; k<8; k++) {
					printf("%c%02x",
					       k==2 ? '"' : ':',
					       ((unsigned char*)&st[i])[k]);
				}
				printf("\"\n");
				i++;
			} else if (memcmp("ifHC", stn[i], 4) == 0) {
				unsigned long long s = st[i];
				s = (s<<32) + st[i+1];
				if (s)
					printf("% 40s %llu\n", stn[i], s);
				i++;
			} else if (strcmp("ifDescr", stn[i]) == 0) {
				int k;
				st[i] = htonl(st[i]);
				st[i+1] = htonl(st[i+1]);
				st[i+2] = htonl(st[i+2]);
				st[i+3] = htonl(st[i+3]);
				st[i+4] = htonl(st[i+4]);
				st[i+5] = htonl(st[i+5]);
				st[i+6] = htonl(st[i+6]);
				st[i+7] = htonl(st[i+7]);
				printf("% 40s \"", "ifDescr");
				for (k=0; k<32; k++) {
					if ((((unsigned char*)&st[i])[k]) == 0)
						break;
					printf("%c", ((unsigned char*)&st[i])[k]);
				}
				printf("\"\n");
				i+=7;
			} else if (strcmp("ifAdminStatus", stn[i]) == 0 ||
				   strcmp("ifOperStatus", stn[i]) == 0) {
				switch (st[i]) {
				case 1:
					printf("% 40s Up(1)\n", stn[i]);
				break;case 2:
					printf("% 40s Down(2)\n", stn[i]);
				break;case 3:
					printf("% 40s Test(3)\n", stn[i]);
				break;case 4:
					printf("% 40s Unknown(4)\n", stn[i]);
				break;case 5:
					printf("% 40s Dormant(5)\n", stn[i]);
				break;default:
					printf("% 40s %u\n", stn[i], st[i]);
				}
			} else if (st[i]) {
				printf("% 40s %u\n", stn[i], st[i]);
			}
		}
	}
	exit(0);

oops:
	fprintf(stderr, "Something is wrong, sorry\n");
	exit(-1);
}
